@@ -1,4 +1,3 @@
-#error COSS Support is not stable yet in Squid-3. Please do not use.
 /*
  * $Id$
  * vim: set et :
@@ -49,6 +48,8 @@
 #include "ConfigOption.h"
 #include "StoreFScoss.h"
 #include "Parsing.h"
+#include "swap_log_op.h"
+//#include "SquidMath.h"
 
 #define STORE_META_BUFSZ 4096
 
@@ -80,8 +81,8 @@ static StoreEntry *storeCossAddDiskRestore(CossSwapDir * SD, const cache_key * k
         time_t timestamp,
         time_t lastref,
         time_t lastmod,
-        u_int32_t refcount,
-        u_int16_t flags,
+        uint32_t refcount,
+        uint16_t flags,
         int clean);
 static void storeCossDirRebuild(CossSwapDir * sd);
 static void storeCossDirCloseTmpSwapLog(CossSwapDir * sd);
@@ -90,13 +91,13 @@ static FILE *storeCossDirOpenTmpSwapLog(CossSwapDir *, int *, int *);
 static char *
 storeCossDirSwapLogFile(SwapDir * sd, const char *ext)
 {
-    LOCAL_ARRAY(char, path, SQUID_MAXPATHLEN);
-    LOCAL_ARRAY(char, pathtmp, SQUID_MAXPATHLEN);
+    LOCAL_ARRAY(char, path, MAXPATHLEN);
+    LOCAL_ARRAY(char, pathtmp, MAXPATHLEN);
     LOCAL_ARRAY(char, digit, 32);
     char *pathtmp2;
 
     if (Config.Log.swap) {
-        xstrncpy(pathtmp, sd->path, SQUID_MAXPATHLEN - 64);
+        xstrncpy(pathtmp, sd->path, MAXPATHLEN - 64);
         pathtmp2 = pathtmp;
 
         while ((pathtmp2 = strchr(pathtmp2, '/')) != NULL)
@@ -106,15 +107,15 @@ storeCossDirSwapLogFile(SwapDir * sd, const char *ext)
             pathtmp[strlen(pathtmp) - 1] = '\0';
 
         for (pathtmp2 = pathtmp; *pathtmp2 == '.'; pathtmp2++);
-        snprintf(path, SQUID_MAXPATHLEN - 64, Config.Log.swap, pathtmp2);
+        snprintf(path, MAXPATHLEN - 64, Config.Log.swap, pathtmp2);
 
-        if (strncmp(path, Config.Log.swap, SQUID_MAXPATHLEN - 64) == 0) {
+        if (strncmp(path, Config.Log.swap, MAXPATHLEN - 64) == 0) {
             strcat(path, ".");
             snprintf(digit, 32, "%02d", sd->index);
             strncat(path, digit, 3);
         }
     } else {
-        xstrncpy(path, sd->path, SQUID_MAXPATHLEN - 64);
+        xstrncpy(path, sd->path, MAXPATHLEN - 64);
         strcat(path, "/swap.state");
     }
 
@@ -200,12 +201,12 @@ CossSwapDir::readCompleted(const char *buf, int len, int errflag, RefCount<ReadR
             cstate->readbuffer = (char *)xmalloc(cstate->st_size);
             p = storeCossMemPointerFromDiskOffset(storeCossFilenoToDiskOffset(sio->swap_filen),
                                                   NULL);
-            xmemcpy(cstate->readbuffer, p, cstate->st_size);
+            memcpy(cstate->readbuffer, p, cstate->st_size);
         }
 
         sio->offset_ += len;
-        xmemcpy(cstate->requestbuf, &cstate->readbuffer[cstate->requestoffset],
-                cstate->requestlen);
+        memcpy(cstate->requestbuf, &cstate->readbuffer[cstate->requestoffset],
+               cstate->requestlen);
         rlen = (size_t) cstate->requestlen;
     }
 
@@ -510,8 +511,8 @@ storeCossAddDiskRestore(CossSwapDir * SD, const cache_key * key,
                         time_t timestamp,
                         time_t lastref,
                         time_t lastmod,
-                        u_int32_t refcount,
-                        u_int16_t flags,
+                        uint32_t refcount,
+                        uint16_t flags,
                         int clean)
 {
     StoreEntry *e = NULL;
@@ -784,8 +785,8 @@ CossCleanLog::write(StoreEntry const &e)
     s.swap_file_sz = e.swap_file_sz;
     s.refcount = e.refcount;
     s.flags = e.flags;
-    xmemcpy(&s.key, e.key, SQUID_MD5_DIGEST_LENGTH);
-    xmemcpy(outbuf + outbuf_offset, &s, ss);
+    memcpy(&s.key, e.key, SQUID_MD5_DIGEST_LENGTH);
+    memcpy(outbuf + outbuf_offset, &s, ss);
     outbuf_offset += ss;
     /* buffered write */
 
@@ -835,10 +836,9 @@ CossSwapDir::writeCleanDone()
     /* rename */
 
     if (state->fd >= 0) {
-#if defined(_SQUID_OS2_) || defined(_SQUID_WIN32_)
+#if _SQUID_OS2_ || _SQUID_WINDOWS_
         file_close(state->fd);
         state->fd = -1;
-
 #endif
 
         xrename(state->newLog, state->cur);
@@ -889,7 +889,7 @@ CossSwapDir::logEntry(const StoreEntry & e, int op) const
     s->swap_file_sz = e.swap_file_sz;
     s->refcount = e.refcount;
     s->flags = e.flags;
-    xmemcpy(s->key, e.key, SQUID_MD5_DIGEST_LENGTH);
+    memcpy(s->key, e.key, SQUID_MD5_DIGEST_LENGTH);
     file_write(swaplog_fd,
                -1,
                s,
@@ -996,8 +996,8 @@ void
 CossSwapDir::statfs(StoreEntry & sentry) const
 {
     storeAppendPrintf(&sentry, "\n");
-    storeAppendPrintf(&sentry, "Maximum Size: %"PRIu64" KB\n", max_size);
-    storeAppendPrintf(&sentry, "Current Size: %"PRIu64" KB\n", cur_size);
+    storeAppendPrintf(&sentry, "Maximum Size: %lu KB\n", max_size);
+    storeAppendPrintf(&sentry, "Current Size: %lu KB\n", cur_size);
     storeAppendPrintf(&sentry, "Percent Used: %0.2f%%\n",
                       (100.0 * (double)cur_size / (double)max_size) );
     storeAppendPrintf(&sentry, "Number of object collisions: %d\n", (int) numcollisions);
@@ -1095,7 +1095,7 @@ CossSwapDir::reconfigure(int index, char *path)
 void
 CossSwapDir::dump(StoreEntry &entry)const
 {
-    storeAppendPrintf(&entry, " %"PRIu64"", (max_size >> 10));
+    storeAppendPrintf(&entry, " %lu", (max_size >> 10));
     dumpOptions(&entry);
 }
 
